import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.util.EntityUtils;


public class PyCodiAPI
{

    public static void main(String[] args) throws Exception
    {
        // select files
        File source = new File("examples/codi/cmt.owl");
        File target = new File("examples/codi/edas.owl");
        File reference = new File("examples/codi/cmt-edas.rdf");

        // String sourceURL = "http://seals.sti2.at/tdrs-web/testdata/persistent/Conference+Testsuite/2010/suite/cmt-confOf/component/source/";
        // String targetURL = "http://seals.sti2.at/tdrs-web/testdata/persistent/Conference+Testsuite/2010/suite/cmt-confOf/component/target/";
        // String referenceURL = "http://seals.sti2.at/tdrs-web/testdata/persistent/Conference+Testsuite/2010/suite/cmt-confOf/component/reference/";

        String id = startCodiFile(source, target, reference, "2");
        // String id = startCodiUrl(sourceURL, targetURL, referenceURL, "2");
        String state = PyExecutor.getAction(id, "state");
        while(state.startsWith("wating")) {
            Thread.currentThread();
            Thread.sleep(2500);
            state = PyExecutor.getAction(id, "state");
            System.out.println(state);
        }
        int position = 0;
        while(state.equals("running")) {
            Thread.currentThread();
            Thread.sleep(2500);
            String log = PyExecutor.getAction(id, "log");
            System.out.println(log.substring(position).trim());
            position = log.length();
            state = PyExecutor.getAction(id, "state");
        }
        System.out.println(state);
        // String state = "complete", id = "9978b811-459b-42c4-b08b-d42e35e5fd2d";
        if(state.equals("complete")) {
            // download file
            File out = new File("res" + System.getProperty("file.separator") + id + ".rdf");
            if(!new File("res").exists()) {
                new File("res").mkdir();
            }
            System.out.println("Result saved to: " + PyExecutor.getResult(id, "alignment", out));

            InputStream in = new FileInputStream(out);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String line;
            while((line = br.readLine()) != null) {
                System.out.println(line);
            }
            in.close();

            System.out.println("\n=============================\nFinish => " + out.getAbsolutePath());
        }
    }


    private synchronized static String startCodiFile(File source, File target, File reference, String setting) throws Exception
    {
        HttpClient httpclient = new DefaultHttpClient();
        httpclient.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);

        HttpPost httppost = new HttpPost(PyExecutor.server + "startCODI");

        // build entity:
        MultipartEntity mpEntity = new MultipartEntity();
        mpEntity.addPart("source", new FileBody(source, "multipart"));
        mpEntity.addPart("target", new FileBody(target, "multipart"));
        if(reference != null) {
            mpEntity.addPart("reference", new FileBody(reference, "multipart"));
        }
        mpEntity.addPart("setting", new StringBody(setting));
        httppost.setEntity(mpEntity);

        System.out.println("executing request " + httppost.getRequestLine());
        HttpResponse response = httpclient.execute(httppost);
        HttpEntity resEntity = response.getEntity();

        System.out.println("status: " + response.getStatusLine());
        String result = null;
        if(resEntity != null) {
            result = EntityUtils.toString(resEntity);
            System.out.println("ID " + result);
            EntityUtils.consume(resEntity);
        }
        httpclient.getConnectionManager().shutdown();
        return result;
    }


    public synchronized static String startCodiUrl(String sourceURL, String targetURL, String referenceURL, String setting) throws Exception
    {
        HttpClient httpclient = new DefaultHttpClient();
        httpclient.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);
        // httpclient.getParams().setParameter("http.socket.timeout", new Integer(20000));

        HttpPost httppost = new HttpPost(PyExecutor.server + "startCODI");

        // build entity:
        MultipartEntity mpEntity = new MultipartEntity();
        mpEntity.addPart("source", new StringBody(sourceURL));
        mpEntity.addPart("target", new StringBody(targetURL));
        mpEntity.addPart("reference", new StringBody(referenceURL));
        mpEntity.addPart("setting", new StringBody(setting));
        httppost.setEntity(mpEntity);

        System.out.println("executing request " + httppost.getRequestLine());
        HttpResponse response = httpclient.execute(httppost);
        HttpEntity resEntity = response.getEntity();

        System.out.println("status: " + response.getStatusLine());
        String result = null;
        if(resEntity != null) {
            result = EntityUtils.toString(resEntity);
            System.out.println("ID " + result);
            EntityUtils.consume(resEntity);
        }
        httpclient.getConnectionManager().shutdown();
        return result;
    }
}
